(function() {
  var App, Output, SyncPrompt, commands,
    bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  SyncPrompt = require('./sync_prompt');

  Output = require('./output/local_output');

  commands = require('./commands');

  App = (function() {
    App.prototype._commands = [];

    function App(scope) {
      this.scope = scope;
      this.find_command = bind(this.find_command, this);
      this.typeahead = bind(this.typeahead, this);
      this.output = new Output();
      this.prompt = new SyncPrompt({
        typeahead: this.typeahead
      });
      this.prompt.on('data', this.find_command);
    }

    App.prototype.commands = function() {
      var _, command;
      if (this._commands.length === 0) {
        for (_ in commands) {
          command = commands[_];
          this._commands.push(new command({
            output: this.output,
            scope: this.scope,
            prompt: this.prompt
          }));
        }
      }
      return this._commands;
    };

    App.prototype.typeahead = function(input) {
      var command, i, items, len, ref;
      if (input == null) {
        input = '';
      }
      items = [];
      ref = this.commands();
      for (i = 0, len = ref.length; i < len; i++) {
        command = ref[i];
        items = items.concat(command.typeahead(input));
      }
      if (input) {
        items = items.filter(function(item) {
          return item.indexOf(input) === 0;
        });
      }
      return [items, input];
    };

    App.prototype.find_command = function(input, chain) {
      var args, command, i, len, match, ref;
      ref = this.commands();
      for (i = 0, len = ref.length; i < len; i++) {
        command = ref[i];
        if (match = command.match(input.trim())) {
          args = String(match[1]).trim().split(' ');
          return command.execute.call(command, args, chain);
        }
      }
      return false;
    };

    App.prototype.open = function() {
      this.prompt.type('whereami');
      return this.prompt.open();
    };

    return App;

  })();

  module.exports = App;

}).call(this);
